/*****************************************************************************
ETHAN 1.0 

Automated detection of spherical particles from electron micrographs
Copyright (C) 2000 Teemu Kivioja (Teemu.Kivioja@cs.Helsinki.FI)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.
*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>
#include <string.h>
#include <ctype.h>


#include "memalloc.h"


#define YES              1
#define NO               0
#define DELETED         -1
#define DRAWING_VALUE    1

#define ETHAN_MEMORY_ERROR     1      /* malloc failed */
#define ETHAN_READ_ERROR       2      /* opening or reading file failed */
#define ETHAN_WRITE_ERROR      3      /* writing file failed */
#define ETHAN_PARAMETER_ERROR  4      /* illegal parameters given to the */
                                      /* program on command line */
#define ETHAN_FORMAT_ERROR     5      /* program could not read the */ 
				      /* file format */
                                        

/* structure to hold possible virus center */
struct peak {
    int x;                  /* x coordinate */
    int y;                  /* y coordinate */
    int height;             /* peak height in filtered image */ 
    int best;               /* the ring filter which gives heighest peak */
};


extern float height_param;
extern float square_width_param;
extern float ring_width_param;
extern float dist_param;
extern float count_param;

extern float radius_coff;

extern int size;
extern int border;
extern int file_number;

extern int sector_test;

void find_viruses(int r, char *input_filename, char *coord_filename, 
                  char *dirname, int sample_rate);


FILE *open_mrc(char *filename, int *m, int *n, int *item_size,
               int *header_size);

unsigned char **read_sampled_piece_c(char *filename, FILE *fp, int header_size,
                                     int width, int height, int piece_width, 
                                     int start_x, int start_y, 
                                     int sampled_width, int sampled_height, 
                                     int s_rate, int item_size, 
                                     unsigned char **image, 
                                     unsigned char *rowp);


FILE *open_pgm(char *filename, int *width, int *height, int *item_size, 
               int *header_size);

void write_piece_ascii_c(FILE *fp, int **image, int width, 
                         int height);

FILE *ppm_write_open(char *filename, int w, int h, int sigma);


void write_piece_ppm_c(FILE *fp, unsigned char **image, 
                       unsigned char **circle_image, int width, int height);


FILE *mrc_write_open(char *filename, int nx, int ny, int nz);

int *calculate_edge(int r);


void edit_peaks(int limit, int n_squares_x, int n_squares_y,
                  struct peak *peaks);


void delete_peaks(int limit, int n_squares_x, int n_squares_y,
                  struct peak *peaks);


void check_peaks(struct peak *peaks, int k);


void find_peaks(int square_width, int first_x, int first_y,
                int dx, int dy, int **filtered_image, struct peak *peaks);



void refine_and_cut(char *filename, char *dir_name, 
                    FILE *fp, int header_size, int width, 
                    int height, int sample_rate, int rb, int rs, 
                    struct peak *peaks, int k);



unsigned char **read_averaged_piece(char *filename, FILE *fp, int header_size,
                                    int width, int height, int piece_width_o, 
                                    int start_x, int start_y, 
                                    int sampled_width, int sampled_height, 
                                    int s_rate, int item_size,
                                    unsigned char **image);

void test_q(int rb, int rs, int width, int height, int n, 
            unsigned char **image, struct peak *peaks);  

struct peak max_filter(int rb, int rs, int start_x, int start_y, int width, 
                       int height, unsigned char **piece);

struct peak find_max(int ix, int iy, int lx, int ly, int **image);

void donut_filter_c(int rb, int rs, int fstart_x, int fstart_y, int m, int n, 
                    unsigned char **image, int **filtered_image);    


void determine_radii(int rb, int rs, int *rbt, int *rst);


void write_mrc_particle(char *filename, char *dir_name, FILE *fp, 
                        int header_size, int width, int height, struct peak p);
